;; nxml-menu.el --- All-in-one menu for nXML mode
;; $Id: nxml-menu.el,v 1.5 2003/11/15 08:54:52 smith Exp $
;; $Date: 2003/11/15 08:54:52 $
;;
;;; Commentary
;;
;; Adds a menu for nXML mode to your Emacs menubar.  The submenus
;; and menu items look like this:
;;
;;   nXML
;;     Modify >
;;     Move   >
;;     Markup >
;;     Show   >
;;     ------
;;     Toggle nXML Validate Mode
;;     ------
;;     Associate RNG Schema with Current Document Automatically
;;     Associate RNG Schema with Current Document Manually
;;     Associate RNG Schema with Current Document By Type
;;     ------
;;     Show RNG Schema Associated with Current Document
;;     Save RNG Schema Association for Future Sessions
;;     ------
;;     Goto First Error
;;     Goto Next Error
;;     Goto Previous Error
;;     ------
;;     Customize nXML Options
;;     Show nXML Version
;;
;; It also defines two new functions, `nxml-kill-element-forward' and
;; `nxml-kill-element-backward', and binds those to menu items and to
;; the key sequences "C-c C-k" and "C-c C-backspace".
;; 
;; To activate the menus, drop the nxml-menu.el file and accompanying
;; "nxml-manual" directory into the same directory where you have your
;; nxml-mode lisp files.  Then add the following to your .emacs
;;
;;   (require 'nxml-menu)
;;
;; The included "nxml-manual" directory contains an HTML version of
;; the nXML manual, generated from the texinfo version in the nXML
;; distribution. If you want to be able to access the HTML version of
;; the manual from the nXML menu, you need to tell nxml-menu where the
;; manual directory is. You can do that either by going to the nXML
;; menu and choosing "Customize nXML Options" and then setting a value
;; for the "Nxml Manual Directory" option, or by specifying a value
;; for the `nxml-manual-directory' variable in your .emacs file.
;; For example:
;;
;;   (setq nxml-manual-directory "/usr/doc/nxml-mode")
;;
;;; Changes
;;
;; 2003-11-15
;;
;;   Changed mechanism for the finding the HTML version of the nXML
;;   manual, partly because I think it might have been the cause for a
;;   bug that Bruce D'Arcus reported, and partly just to make it
;;   customizable (so users can put the manual wherever they want).


;;; Code

(require 'easymenu)

;; The following are used only for finding the generated HTML files
;; for the HTML version of the nXML manual.
(defcustom nxml-manual-directory ""
  "Directory that contains the HTML version of the nxml manual."
  :type 'string
  :group 'nxml
  :require 'nxml-mode
  )

(defvar nxml-menu-definition-modify
  (list "Modify"
	["Indent Region"   indent-region t]
	["Fill Paragraph"  fill-paragraph t]
	["Mark Paragraph"  nxml-mark-paragraph t]
	["------" t]
	["Kill Element Forward" nxml-kill-element-forward t]
	["Kill Element Backward"  nxml-kill-element-backward t]
	["-----" t]
	["Kill Tag Forward"  kill-sexp t]
	["Kill Tag Backward"  backward-kill-sexp t]
	)
  "nXML Modify menu definition.")

(defvar nxml-menu-definition-move
  (list "Move"
	["Forward Paragraph"    nxml-forward-paragraph t]
	["Backward Paragraph"   nxml-backward-paragraph t]
	["----" t]
	["Forward Element" nxml-forward-element t]
	["Backward Element"  nxml-backward-element t]
	["Down Through Current Element Tree"  nxml-down-element t]
	["Up Through Current Element Tree"   nxml-backward-up-element t]
	)
  "nXML Move menu definition.")

(defvar nxml-menu-definition-markup
  (list "Markup"
	["Insert Log Event"
	 (nxml-mode-insert-template "/docbook/log-event.xml" "log-event" "refentry id='") t]
	["Insert Config Key"
	 (nxml-mode-insert-template "/docbook/config-key.xml" "config-key" "refentry id='") t]
	["Insert Procedure"
	 (nxml-mode-insert-template "/docbook/procedure.xml" "procedure" "<para>") t]
	["Insert Figure"
	 (nxml-mode-insert-template "/docbook/figure.xml" "figure" "fileref='") t]
	["---" t]
	["Complete in Current Context" nxml-complete t]
	["Insert Closing Tag (Block)" nxml-balanced-close-start-tag-block t]
	["Insert Closing Tag (Inline)" nxml-balanced-close-start-tag-inline t]
	["Finish Current Context" nxml-finish-element t]
	["----" t]
	["Apply Inferred Markup to Word Before Point" nxml-dynamic-markup-word t]
	["-----" t]
	["Insert Character Entity Reference by Full Name" nxml-insert-named-char t]
	["Insert Character by Full Name" (nxml-insert-named-char (universal-argument)) t]
	["------" t]
	["Insert XML Declaration" nxml-insert-xml-declaration t]
)
  "nXML Markup menu definition.")

(defvar nxml-menu-definition-show
  (list "Show"
	["Show this XML File in Web Browser"
	 (browse-url (concat "file:///" (buffer-file-name))) t]
	["Show Glyphs for Character Entity References"
	 nxml-toggle-char-ref-extra-display t]
	["-----" t]
	["Show nXML Manual (Info)" nxml-mode-visit-info-file t]
	(list "Show nXML Manual (HTML)"
	      ["Completion"
	       (browse-url (concat "file:///" nxml-manual-directory "/Completion.html")) t]
	      ["Inserting end-tags"
	       (browse-url (concat "file:///" nxml-manual-directory "Inserting-end-tags.html")) t]
	      ["Paragraphs"
	       (browse-url (concat "file:///" nxml-manual-directory "/Paragraphs.html")) t]
	      (list "Locating a schema"
		    ["Intro"
		     (browse-url (concat "file:///" nxml-manual-directory "/Locating-a-schema.html")) t]
		    ["Commands for locating a schema"
		     (browse-url (concat "file:///" nxml-manual-directory "/Commands-for-locating-a-schema.html")) t]
	      (list "Schema locating files"
		    ["Intro"
		     (browse-url (concat "file:///" nxml-manual-directory "/Schema-locating-files.html")) t]
		    ["Schema locating file syntax basics"
		     (browse-url (concat "file:///" nxml-manual-directory "/Schema-locating-file-syntax-basics.html")) t]
		    ["Using the document's URI to locate a schema"
		     (browse-url (concat "file:///" nxml-manual-directory "/Using-the-document-s-URI-to-locate-a-schema.html")) t]
		    ["Using the document element to locate a schema"
		     (browse-url (concat "file:///" nxml-manual-directory "/Using-the-document-element-to-locate-a-schema.html")) t]
		    ["Using type identifiers in schema locating files"
		     (browse-url (concat "file:///" nxml-manual-directory "/Using-type-identifiers-in-schema-locating-files.html")) t]
		    ["Using multiple schema locating files"
		     (browse-url (concat "file:///" nxml-manual-directory "/Using-multiple-schema-locating-files.html")) t]
		    )
	      )
	      ["DTDs"
	       (browse-url (concat "file:///" nxml-manual-directory "/DTDs.html")) t]
	      ["Limitations"
	       (browse-url (concat "file:///" nxml-manual-directory "/Limitations.html")) t]
	      )
	)
  "nXML Show menu definition.")

(defvar nxml-menu-definition-nxml
  (list "nXML"
	nxml-menu-definition-modify
	nxml-menu-definition-move
	nxml-menu-definition-markup
	nxml-menu-definition-show
	["--" t]
	["Toggle nXML Validate Mode"   rng-validate-mode t]
	["---" t]
	["Associate RNG Schema with Current Document Automatically"  rng-auto-set-schema t]
	["Associate RNG Schema with Current Document Manually" rng-set-schema-and-validate t]
	["Associate RNG Schema with Current Document By Type" rng-set-document-type-and-validate t]
	["-----" t]
	["Show RNG Schema Associated with Current Document" rng-what-schema t]
	["Save RNG Schema Association for Future Sessions"  rng-save-schema-location t]
	["------" t]
	["Goto First Error" rng-first-error t]
	["Goto Next Error"  rng-next-error t]
	["Goto Previous Error" rng-previous-error t]
	["-------" t]
	["Customize nXML Options" (customize-group 'nxml) t]
	["Show nXML Version" nxml-version t]
	)
  "nXML nXML menu definition.")

(add-hook 'nxml-mode-hook
	  (lambda () 
	    (easy-menu-define 
	      nxml-menu-nxml
	      nxml-mode-map
	      "nXML"
	      nxml-menu-definition-nxml)
	    )
	  )

(defun nxml-mode-visit-info-file ()
  (interactive)
  "Visit the nxml info file."
  (require 'info)
  (Info-find-node "nxml-mode.info" "Top")
  )

(defun nxml-kill-element-forward ()
  (interactive)
  "Kill the element (and all its child content) after point."
  (let ((beg (point))) (nxml-forward-element) (delete-region beg (point)))
  )

(defun nxml-kill-element-backward ()
  (interactive)
  "Kill the element (and all its child content) before point."
  (let ((beg (point))) (nxml-backward-element) (delete-region beg (point)))
  )

(defun nxml-mode-nxml-menu-additional-keys ()
  "Defines bindings for key bindings added by nxml-menu."
  (define-key nxml-mode-map "\C-c\C-k" 'nxml-kill-element-forward)
  (define-key nxml-mode-map [?\C-c \C-backspace] 'nxml-kill-element-backward)
  )

(add-hook 'nxml-mode-hook 'nxml-mode-nxml-menu-additional-keys)

(provide 'nxml-menu)

;;; nxml-menu.el ends here